/*
 Wireless Gameboy Controller TX
 Version: 1.2
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 23/09/2018
 Last Modified: 28/10/2018
 
 Use your Gameboy, GBC, GBP, GBA or GBA SP as a Wireless game controller for any device which supports USB HID Joysticks.
 This would allow you to play games on your PC with your a Gameboy of your choosing.
 
 Program the ATmega48PA to use an external crystal (16MHz) and divide clock by 1
 avrdude -p ATmega48p -c usbasp -U lfuse:w:0xEF:m -U hfuse:w:0xDF:m -U efuse:w:0xFF:m
 
*/

// ATmega48P Pin map
// -------------------------------------
// PB0 - LED			PC0 - A12		PD0 - D0
// PB1 - nRF CE		PC1 - A13		PD1 - D1
// PB2 - nRF CSN		PC2 - A14		PD2 - D2
// PB3 - nRF MOSI		PC3 - A15		PD3 - D3
// PB4 - nRF MISO		PC4 - WR			PD4 - D4
// PB5 - nRF SCK							PD5 - D5
//												PD6 - D6
//												PD7 - D7


#define F_CPU 16000000 // 16 MHz

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include "setup.c"
#include "mirf.c"


int main(void) {
	setup();
	
	NRF_TX_POWERUP; // Power up to transmitter mode
	_delay_ms(5);
	mirf_flush_rx_tx(); // Flush RX and TX FIFO
	
	uint8_t kpAddressMatch = 0;
	uint8_t configStage = 0;
	
	while(1) {
		// Wait for WR to go low
		while (PINC & 0x10);
		uint8_t readAddr = PINC; // Read address
		asm volatile("nop");
		uint8_t readData = PIND; // Read data
		
		if ((readAddr & 0x1F) == 0x07) { // Address 0x7000 and WR low
			PORTB |= (1<<LED);
			dataOut[0] = readData;
			for (uint8_t t = 0; t < nrfPpr; t++) {
				mirf_transmit_data(1); // Transmit single data byte
			}
			PORTB &= ~(1<<LED);
			kpAddressMatch = 1;
		}
		
		// Check for new config updates
		if (kpAddressMatch == 0) { // Only check 0x6000 address if we haven't received 0x7000 at start up, protects the EEPROM from being re-written (just in case).
			if ((readAddr & 0x1F) == 0x06) { // 0x6000 and WR low
				if (configStage == 0) { // RF Channel (first stage)
					nrfChannel = readData;
					configStage++;
				}
				else if (configStage == 1) { // RF address (second stage)
					nrfTx[4] = readData;
					configStage++;
				}
				else if (configStage == 2) { // Packets per key press (third stage)
					PORTB |= (1<<LED);
					nrfPpr = readData;
					
					// Update the EEPROM
					eeprom_write_byte((uint8_t*) nrfChannelLocation, nrfChannel);
					eeprom_write_byte((uint8_t*) nrfTxLocation, nrfTx[4]);
					eeprom_write_byte((uint8_t*) nrfPprLocation, nrfPpr);
					
					// Update the nRF settings to transmit the data to the RX
					mirf_config_setting_update();
					
					// Transmit the nrf setting to the receiver until powered off (no need to transmit PPK)
					dataOut[0] = nrfChannel;
					dataOut[1] = nrfTx[4];
					while (1) {
						mirf_transmit_data(2);
						_delay_ms(50);
					}
				}
			}
		}
	}
}
